@@ -24,6 +24,7 @@
 #include "vtkSMSessionProxyManager.h"
 #include "vtkSMSourceProxy.h"
 #include "vtkStreamingDemandDrivenPipeline.h"
+#include "vtksys/SystemTools.hxx"
 
 #if VTK_MODULE_ENABLE_VTK_ParallelMPI
 #include "vtkMPI.h"
@@ -37,6 +38,12 @@
 #include "vtkFidesReader.h"
 #endif
 
+#if defined(_WIN32) && !defined(__MINGW32__)
+const char SPLIT_PATH_CHAR = ';';
+#else
+const char SPLIT_PATH_CHAR = ':';
+#endif
+
 #include "catalyst_impl_paraview.h"
 
 static bool update_producer_mesh_blueprint(const std::string& channel_name,
@@ -241,6 +248,7 @@ enum catalyst_status catalyst_initialize_paraview(const conduit_node* params)
   {
     vtkLogF(
       ERROR, "invalid 'catalyst' node passed to 'catalyst_initialize'. Initialization failed.");
+    // NOLINTNEXTLINE(clang-analyzer-optin.core.EnumCastOutOfRange)
     return pvcatalyst_err(invalid_node);
   }
 
@@ -260,7 +268,13 @@ enum catalyst_status catalyst_initialize_paraview(const conduit_node* params)
 #else
   const vtkTypeUInt64 comm = 0;
 #endif
-  vtkInSituInitializationHelper::Initialize(comm);
+  std::vector<std::string> python_paths;
+  if (cpp_params.has_path("catalyst/python_path"))
+  {
+    std::string pythonPath = cpp_params["catalyst/python_path"].as_string();
+    python_paths = vtksys::SystemTools::SplitString(pythonPath, SPLIT_PATH_CHAR);
+  }
+  vtkInSituInitializationHelper::Initialize(comm, python_paths);
 
   if (cpp_params.has_path("catalyst/scripts"))
   {
@@ -348,6 +362,7 @@ enum catalyst_status catalyst_execute_paraview(const conduit_node* params)
   if (!cpp_params.has_path("catalyst"))
   {
     vtkVLogF(PARAVIEW_LOG_CATALYST_VERBOSITY(), "Path 'catalyst' is not provided. Skipping.");
+    // NOLINTNEXTLINE(clang-analyzer-optin.core.EnumCastOutOfRange)
     return pvcatalyst_err(invalid_node);
   }
 
@@ -355,6 +370,7 @@ enum catalyst_status catalyst_execute_paraview(const conduit_node* params)
   if (!vtkCatalystBlueprint::Verify("execute", root))
   {
     vtkLogF(ERROR, "invalid 'catalyst' node passed to 'catalyst_execute'. Execution failed.");
+    // NOLINTNEXTLINE(clang-analyzer-optin.core.EnumCastOutOfRange)
     return pvcatalyst_err(invalid_node);
   }
 
@@ -582,5 +598,6 @@ enum catalyst_status catalyst_results_paraview(conduit_node* params)
 
   vtkInSituInitializationHelper::GetResultsFromPipelines(params);
 
+  // NOLINTNEXTLINE(clang-analyzer-optin.core.EnumCastOutOfRange)
   return is_success ? catalyst_status_ok : pvcatalyst_err(results);
 }
