@@ -21,12 +21,14 @@
 #include "vtkSMPTools.h"
 #include "vtkStreamingDemandDrivenPipeline.h"
 #include "vtkVector.h"
-#include "vtkVectorOperators.h"
 
 #include "vtksys/FStream.hxx"
 #include "vtksys/SystemTools.hxx"
 
-#include "nlohmann/json.hpp"
+#include "vtk_nlohmannjson.h"
+// clang-format off
+#include VTK_NLOHMANN_JSON(json.hpp)
+// clang-format on
 
 #include <algorithm>
 #include <cmath>
