@@ -290,7 +290,8 @@ int vtkImageVolumeRepresentation::RequestData(
     {
       auto images = vtkCompositeDataSet::GetDataSets(doTree);
       images.erase(std::remove_if(images.begin(), images.end(),
-                     [](vtkDataSet* ds) {
+                     [](vtkDataSet* ds)
+                     {
                        return vtkRectilinearGrid::SafeDownCast(ds) == nullptr &&
                          vtkImageData::SafeDownCast(ds) == nullptr;
                      }),
@@ -341,6 +342,11 @@ bool vtkImageVolumeRepresentation::AddToView(vtkView* view)
   if (rview)
   {
     rview->GetRenderer()->AddActor(this->Actor);
+
+    // Indicate that the above renderer is the one the actor is relative to
+    // in case the coordinate system is set to physical or device.
+    this->Actor->SetCoordinateSystemRenderer(rview->GetRenderer());
+
     // Indicate that this is a prop to be rendered during hardware selection.
     rview->RegisterPropForHardwareSelection(this, this->GetRenderedProp());
 
@@ -355,6 +361,7 @@ bool vtkImageVolumeRepresentation::RemoveFromView(vtkView* view)
   vtkPVRenderView* rview = vtkPVRenderView::SafeDownCast(view);
   if (rview)
   {
+    this->Actor->SetCoordinateSystemRenderer(nullptr);
     rview->GetRenderer()->RemoveActor(this->Actor);
     return this->Superclass::RemoveFromView(view);
   }
@@ -378,6 +385,8 @@ void vtkImageVolumeRepresentation::UpdateMapperParameters()
   if (this->UseSeparateOpacityArray)
   {
     // See AppendOpacityComponent() for the construction of this array.
+    // FIXME: `colorArrayName` might be `nullptr` here.
+    // NOLINTNEXTLINE(clang-analyzer-cplusplus.StringChecker)
     std::string combinedName(colorArrayName);
     combinedName += "_and_opacity";
     this->VolumeMapper->SelectScalarArray(combinedName.c_str());
@@ -529,6 +538,12 @@ void vtkImageVolumeRepresentation::SetAnisotropy(float val)
   this->Property->SetScatteringAnisotropy(val);
 }
 
+//----------------------------------------------------------------------------
+void vtkImageVolumeRepresentation::SetCoordinateSystem(int coordSys)
+{
+  this->Actor->SetCoordinateSystem(static_cast<vtkProp3D::CoordinateSystems>(coordSys));
+}
+
 //----------------------------------------------------------------------------
 void vtkImageVolumeRepresentation::SetGlobalIlluminationReach(float val)
 {
