@@ -13,6 +13,7 @@
 #include "vtkDoubleArray.h"
 #include "vtkEventForwarderCommand.h"
 #include "vtkFloatArray.h"
+#include "vtkGarbageCollector.h"
 #include "vtkGenericCell.h"
 #include "vtkGridSynchronizedTemplates3D.h"
 #include "vtkImageData.h"
@@ -152,7 +153,7 @@ void vtkCutter::StructuredPointsCutter(vtkDataSet* dataSetInput, vtkPolyData* th
   this->SynchronizedTemplates3D->ComputeScalarsOff();
   this->SynchronizedTemplates3D->ComputeNormalsOff();
   output = this->SynchronizedTemplates3D->GetOutput();
-  this->SynchronizedTemplatesCutter3D->SetGenerateTriangles(this->GetGenerateTriangles());
+  this->SynchronizedTemplates3D->SetGenerateTriangles(this->GetGenerateTriangles());
   this->SynchronizedTemplates3D->Update();
   output->Register(this);
 
@@ -311,7 +312,8 @@ int vtkCutter::RequestData(
   }
 
   vtkPlane* plane = vtkPlane::SafeDownCast(this->CutFunction);
-  auto executePlaneCutter = [&]() {
+  auto executePlaneCutter = [&]()
+  {
     if (this->Locator == nullptr)
     {
       this->CreateDefaultLocator();
@@ -330,8 +332,7 @@ int vtkCutter::RequestData(
     {
       // Create a copy of vtkPlane and nudge it by the single contour
       vtkNew<vtkPlane> newPlane;
-      newPlane->SetNormal(plane->GetNormal());
-      newPlane->SetOrigin(plane->GetOrigin());
+      newPlane->DeepCopy(plane);
       // Evaluate the distance the origin is from the original plane. This accommodates
       // subclasses of vtkPlane that may have an additional offset parameter not
       // accessible through the vtkPlane interface. Use this distance to adjust the origin
@@ -886,6 +887,7 @@ void vtkCutter::UnstructuredGridCutter(vtkDataSet* input, vtkPolyData* output)
           vtkIdType cellId = cellIter->GetCellId();
           input->SetCellOrderAndRationalWeights(cellId, cell);
           cellIds = cell->GetPointIds();
+          cellScalars->SetNumberOfTuples(numCellPts);
           cutScalars->GetTuples(cellIds, cellScalars);
           // Loop over all contour values.
           for (iter = 0; iter < numContours && !abortExecute; iter++)
@@ -977,6 +979,7 @@ void vtkCutter::UnstructuredGridCutter(vtkDataSet* input, vtkPolyData* output)
           // Fetch the full cell -- most expensive.
           cellIter->GetCell(cell);
           input->SetCellOrderAndRationalWeights(cellId, cell);
+          cellScalars->SetNumberOfTuples(numCellPts);
           cutScalars->GetTuples(pointIdList, cellScalars);
           // Loop over all contour values.
           for (contourIter = contourValues; contourIter != contourValuesEnd; ++contourIter)
@@ -1076,4 +1079,19 @@ void vtkCutter::PrintSelf(ostream& os, vtkIndent indent)
 
   os << indent << "Precision of the output points: " << this->OutputPointsPrecision << "\n";
 }
+
+//------------------------------------------------------------------------------
+void vtkCutter::ReportReferences(vtkGarbageCollector* collector)
+{
+  this->Superclass::ReportReferences(collector);
+  // These share our input and might participate in a reference loop
+  vtkGarbageCollectorReport(collector, this->SynchronizedTemplates3D, "SynchronizedTemplates3D");
+  vtkGarbageCollectorReport(
+    collector, this->SynchronizedTemplatesCutter3D, "SynchronizedTemplatesCutter3D");
+  vtkGarbageCollectorReport(
+    collector, this->GridSynchronizedTemplates, "GridSynchronizedTemplates");
+  vtkGarbageCollectorReport(
+    collector, this->RectilinearSynchronizedTemplates, "RectilinearSynchronizedTemplates");
+  vtkGarbageCollectorReport(collector, this->PlaneCutter, "PlaneCutter");
+}
 VTK_ABI_NAMESPACE_END
